/* win32run.c - December 12, 1997

  Simple wrapper to start Win32 GrADS executables. It must reside in the
directory Win32 GrADS was installed. You must copy this executable to
the following names, e.g.,

"gradsnc"       to run     gradsnc.dll

All command line arguments will be passed to the child process.
It is assumed a standard Win32 GrADS directory structure. If
Win32 GrADS is installed under //progra~1/pcgrads then:

GADDIR = //progra~1/pcgrads/dat
GASCRP = //progra~1/pcgrads/lib
DISPLAY = localhost:0.0

These environment variables are set iff they are not already defined.
The actual grads root directory is derived from the full pathname of the
executable, assuming the executable is under c:/progra~1/pcgrads/win32.

NOTES:

   1. This is a Windows application; is not meant to be portable.
   2. When running Win32e GrADS, executables are started under rxvt

    (c) 1997-2001 Arlindo da Silva
 
    Permission is granted to any individual or institution to use,
    copy, modify, or redistribute this software so long as it is not
    sold for profit, and provided this notice is retained.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <process.h>
#include <Windows.h>
// #include <Windows32/Functions.h>

static void setargv (int *argcPtr, char ***argvPtr);

#ifdef CONSOLE // Console application

int
main(int argc, char **argv)

#else         // Windows application

int APIENTRY
WinMain(hInstance, hPrevInstance, lpszCmdLine, nCmdShow)
    HINSTANCE hInstance;
    HINSTANCE hPrevInstance;
    LPSTR lpszCmdLine;
    int nCmdShow;

#endif

{

    FILE *o, *file;
    char * cmdLine;

#ifndef CONSOLE
    char **argv;
    int argc;
#endif

    char garoot[MAX_PATH];
    char gaddir[MAX_PATH];
    char gatmp[MAX_PATH];
    char gapath[MAX_PATH];
    char gagrx[MAX_PATH];
    char gagui[MAX_PATH];
    char gascrp[MAX_PATH];
    char gaudft[MAX_PATH];
    char display[MAX_PATH];
    char gaexec[MAX_PATH];
    char appl[MAX_PATH];
    char bin[MAX_PATH];
    char cmdl[MAX_PATH];
    char args[MAX_PATH];
    char here[MAX_PATH];
    char tmp[MAX_PATH];
    int i, ia, j, slash=0;
    char win32e=0;

//  Retrieve the command line (Win32 API)
//  ------------------------------------
    cmdLine = GetCommandLine();

#ifndef CONSOLE
     setargv(&argc, &argv);
#endif

//  Find GrADS root directory
//  -------------------------
    GetModuleFileName(NULL, tmp, MAX_PATH);   // Windows native call
    GetShortPathName( tmp, cmdl, MAX_PATH);
    strcpy(garoot,cmdl);
    for(i=strlen(garoot)-1, ia=0; i>-1; i--)
    {
       if(garoot[i]=='/'||garoot[i]=='\\') slash++;
       if(slash==1 && !ia ) {
          ia = i+1;
          garoot[i] = '\0';
          break;
       }
    }


//  Find directory where this executable lives
//  ------------------------------------------
    strcpy(here,cmdl);
    here[ia-1] = '\0';


//  Find application name (short win32 name)
//  ---------------------------------------
    for(i=ia, j=0; i<strlen(cmdl); i++, j++) {
        if(cmdl[i]=='.') break;
        appl[j] = cmdl[i];
    }
    appl[j] = '\0';

// Find name of bin dir (win32 or win32e)
// --------------------------------------
    for(i=strlen(garoot)-6, j=0; i<strlen(garoot); i++, j++) {
        bin[j] = garoot[i];
    }
    bin[j] = '\0';
    if ( !strcmp(bin,"win32e") || !strcmp(bin,"WIN32E") ) win32e = 1;

//  Default environment strings
//  ---------------------------
    sprintf(gaddir,"GADDIR=%s/../dat", garoot);
    sprintf(gascrp,"GASCRP=%s/../lib", garoot);
    sprintf(gagui, "GAGUI=%s/../lib/sample2.gui", garoot);
    sprintf(gaudft,"GAUDFT=%s/../udf/udft", garoot);
    sprintf(gatmp,"TMPDIR=%s/../tmp", garoot);
    if ( win32e ) {
      sprintf(gagrx,"GRXFONT=%s/../dat/GRXFONTS", garoot);
    } else {
      sprintf(display,"DISPLAY=localhost:0.0");
    }

//  Set/reset environment variables
//  -------------------------------
    if (getenv("GADDIR")==NULL)      putenv(gaddir);
    if (getenv("GASCRP")==NULL)      putenv(gascrp);
    // if (getenv("GAUDT")==NULL)    putenv(gaudft);
    if (getenv("TMPDIR")==NULL)      putenv(gatmp);
    if ( win32e ) {
      if (getenv("GRXFONT")==NULL)     putenv(gagrx);
    } else {
      if (getenv("DISPLAY")==NULL)     putenv(display);
    }

//  Make sure current directory is part of path
//  -------------------------------------------
    sprintf(gapath,"PATH=.:%s",getenv("PATH"));
    putenv(gapath);

//  Some application names are aliased
//  ----------------------------------
    if ( !strcmp(appl,"Grads") || !strcmp(appl,"grads") 
                               || !strcmp(appl,"GRADS") ) {
          strcpy ( appl, "GradsDODS" ); // this may not exist!
          sprintf(gaexec,"%s\\%s.dll", here, appl);
          file = fopen(gaexec,"r+b");
          if ( file == (FILE *)NULL ) {
               strcpy ( appl, "GradsNC" ); // try classic
          } else {
               fclose(file);
          }
    }

      if ( !strcmp(appl,"GradsGUI") || !strcmp(appl,"GRADSGUI") ) {
         if (getenv("GAGUI")==NULL)  putenv(gagui);
         strcpy(appl,"GradsDODS");
    }


//  Full path name of application
//  -----------------------------
    if ( win32e ) {
      sprintf(gaexec, 
   "%s/rxvt -display :0 -fn fixedsys -fg blue -cr red -sl 2048 -e %s/%s.dll ", 
               here, here, appl);
      for ( i = 1; i<argc; i++ ) { sprintf(gaexec,"%s %s",gaexec,argv[i]); }
    } else {
      sprintf(gaexec,"%s/%s.dll", here, appl);
    }

#ifndef __CYGWIN32__
// Turn / into \ for "DOS" compliancy
    for(i=strlen(gaexec)-1; i>-1; i--)
        if(gaexec[i]=='/') gaexec[i] = '\\';
#else
    for(i=strlen(gaexec)-1; i>-1; i--)
        if(gaexec[i]=='\\') gaexec[i] = '/';
#endif

//  Starting application
//  --------------------
    // printf("Starting %s...\n", gaexec);
    //    system("env");
 
#ifdef __CYGWIN32__
    if ( win32e ) { return system(gaexec); }
    else          { execv(gaexec,argv); }
#else
    // execl(gaexec,cmdLine,NULL);
    // sprintf(gaexec,"%s/%s.dll %s", here, appl, cmdLine);
    // return system(gaexec);
    return(spawnl(_P_WAIT,gaexec,cmdLine,NULL));

#endif
}
